@@ -49,7 +49,14 @@ module Async
 			@any = nil
 		end
 		
+		def dup
+			self.class.new(@io.dup, @reactor)
+		end
+		
 		def resume(*args)
+			# It's possible that the monitor was closed before calling resume.
+			return unless @monitor
+			
 			readiness = @monitor.readiness
 			
 			if @readable and (readiness == :r or readiness == :rw)
